"""
The parts module.

A part is an element that has been placed
in an specific track. We also consider a
part some imaginary things that are 
fulfilling the track regions in which the
user didn't put any media, so we can 
interact with those 'empty' parts to 
obtain 'empty' audio and video frames.
"""
# from yta_editor.track.parts.audio import _AudioPart
# from yta_editor.track.parts.video import _VideoPart
# from yta_editor.track.parts.abstract import NON_LIMITED_EMPTY_PART_END


# __all__ = [
#     '_AudioPart',
#     '_VideoPart',
#     'NON_LIMITED_EMPTY_PART_END'
# ]